[void] [Reflection.Assembly]::LoadWithPartialName("UIAutomationClient")
[void] [Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
[void] [Reflection.Assembly]::LoadWithPartialName("WindowsBase")
[void] [Reflection.Assembly]::LoadWithPartialName("Microsoft.VisualBasic")

## If you haven't ever used the AutoIt COM DLL, this will register
## it first.
& {
    trap {
        regsvr32 /s AutoitX3.dll
	Start-Sleep 1
        $SCRIPT:autoIt = New-Object -COM AutoitX3.Control
        continue
    }

    $SCRIPT:autoIt = New-Object -COM AutoitX3.Control
}


## Determine if a specific window class is active.
## This is helpful when you want a hotkey to only apply to a certain type
## of window, such as a console window. You can determine the window class
## name from the Control-Alt-1 macro below.
function IsClassActive($class)
{
    [Windows.Automation.AutomationElement]::FocusedElement.Current.ClassName -eq $class
}

## Determine if a specific window title is active.
## This is helpful when you want a hotkey to only apply to a certain window
## title, such as Product Studio. You can determine the window title
## name from the Control-Alt-1 macro below.
function IsWindowActive($title)
{
    (Get-Process -Id ([Windows.Automation.AutomationElement]::FocusedElement.Current.ProcessId)).MainWindowTitle -match $title
}

## Gets a handle to the foreground window
function GetForegroundWindow
{
    [Windows.Automation.AutomationElement]::FocusedElement.Current.NativeWindowHandle
}

## Sets a process as the foreground window.
function SetForegroundWindow($process)
{
    ## We're only allowed to activate the foreground window if we have
    ## focus.
    $form = ${Microsoft.BGShell.RawUI}
    $form.Activate()

    Start-Sleep -m 100

    [Microsoft.VisualBasic.Interaction]::AppActivate($process.Id);
}

## Write text to the BGShell output window
function Write-Debug($text)
{
    $form = ${Microsoft.BGShell.RawUI}
    $form.WriteHost($text + "`r`n")
}

## Send plain text to the current application
function SendString([string] $text)
{
    $text = $text.Replace("[","{[}")
    $text = $text.Replace("]","{]}")
    $text = $text.Replace("+","{+}")
    $text = $text.Replace("^","{^}")
    $text = $text.Replace("%","{%}")
    $text = $text.Replace("~","{~}")
    $text = $text.Replace("(","{(}")
    $text = $text.Replace(")","{)}")
    $text = $text -replace '{(?!.})','{{}'
    $text = $text -replace '(?<!{.)}','{}}'

    SendKeys $text
}

## Send meta-keys to the current application
## Key documentation is here:
## http://msdn2.microsoft.com/en-us/library/system.windows.forms.sendkeys.aspx
function SendKeys($keys)
{
    $form = ${Microsoft.BGShell.RawUI}
    $form.WaitForKeyboardAvailable()

    [System.Windows.Forms.SendKeys]::SendWait($keys)
}

## Activate PowerShell if it's loaded, otherwise launch PowerShell
function ActivatePowerShell
{
    $WINDOW_VISIBLE = 2
    $foundWindow = $false

    foreach($powerShell in Get-Process PowerShell)
    {
        if($powerShell.MainWindowHandle -ne 0)
        {
            SetForegroundWindow $powerShell
            $foundWindow = $true
            break;
        }
    }

    if(-not $foundWindow)
    {
        [System.Diagnostics.Process]::Start("powershell")
    }
}

## Open up the Emacs scratch buffer
function ActivateScratch
{
    $emacs = @(Get-Process Emacs)[0]

    if(-not $emacs)
    {
        $emacsClient = [System.Diagnostics.Process]::Start("C:\Program Files\emacs\emacs\bin\emacsclientw.exe")
        $emacsClient.WaitForExit()
        $emacs = @(Get-Process Emacs)[0]
    }

    SetForegroundWindow $emacs

    ## If we're not already in the scratch buffer, switch to it.
    if($emacs.MainWindowTitle -notlike "*scratch*")
    {
        SendKeys '%x'
        SendString 'switch-to-buffer'
        SendKeys '{ENTER}'
        SendString '*scratch*'
        SendKeys '{ENTER}'
    }
}

function DumpActiveWindow
{
    $GLOBAL:activeWindow = [Windows.Automation.AutomationElement]::FocusedElement.Current
    $output = $activeWindow | Format-List | Out-String
    Write-Debug $output
}

function ClearBgShell
{
    $form = ${Microsoft.BGShell.RawUI}
    $form.OutputBox.Text = ""
}

function SayHello
{
    $form = ${Microsoft.BGShell.RawUI}
    $form.Visible = $true
    $form.Activate()
    $form.Topmost = $true 
    [System.Windows.Forms.MessageBox]::Show($form, "Hello World: $hCount")
}

function EditFileInCurrentTab
{
    ## Only edit if we're in Visual Studio
    SendKeys '^%a'
    Start-Sleep -m 100

    SendString 'File.CopyFullPath'
    SendKeys '{ENTER}'

    $razzle = @(Get-Process | ? { $_.MainWindowTitle -like "Administrator*fbl_srv_powershell*" })[0]
    SetForegroundWindow $razzle

    SendString 'sd edit "'
    $autoIt.Send('! ep')
    SendString '"'
    SendKeys '{ENTER}'
}


$keyMapping = @{}
$keyMapping['Control,Alt,H'] = @{ Action = { SayHello } }

$keyMapping['Control,Alt,Q'] = @{ 
    Action = { SendString 'PowerShell Rocks!' } 
}

$keyMapping['Control,Alt,S'] = @{ Action = { ActivatePowerShell } }
$keyMapping['Control,Alt,N'] = @{ Action = { ActivateScratch } }

## Console enhancements
$keyMapping['Control,Y'] = @{
    KeypressHandled = { IsClassActive 'ConsoleWindowClass' };

    ## Console paste
    Action =  { $autoIt.Send('! ep') }
}
$keyMapping['Control,V'] = @{
    KeypressHandled = { IsClassActive 'ConsoleWindowClass' };

    ## Console scroll
    Action =  { $autoIt.Send('! el') }
}
$keyMapping['Control,S'] = @{
    KeypressHandled = { IsClassActive 'ConsoleWindowClass' };

    ## Console search
    Action =  { $autoIt.Send('! ef') }
}
$keyMapping['Control,L'] = @{
    KeypressHandled = { IsClassActive 'ConsoleWindowClass' };

    ## Console clear
    Action =  { SendKeys '{ESC}cls{ENTER}' }
}
$keyMapping['Control,F'] = @{
    KeypressHandled = { IsClassActive 'ConsoleWindowClass' };

    ## Console clear
    Action =  { SendString '| foreach { $_. }'; SendKeys '{LEFT}{LEFT}' }
}
$keyMapping['Control,W'] = @{
    KeypressHandled = { IsClassActive 'ConsoleWindowClass' };

    ## Console clear
    Action =  { SendString '| where { $_. }'; SendKeys '{LEFT}{LEFT}' }
}
$keyMapping['Control,P'] = @{
    KeypressHandled = { IsClassActive 'ConsoleWindowClass' };

    ## Wrap line in brackets
    Action =  { SendKeys '{HOME}{(}{END}{)}' }
}
$keyMapping['Control,R'] = @{
    KeypressHandled = { IsClassActive 'ConsoleWindowClass' };

    ## History Search
    Action =  { 
        SendKeys '{ESC}'
        SendString 'Get-History | Select-FilteredObject | % { $_ | Invoke-History }'
        SendKeys '{ENTER}'
    }
}
## Console enhancements
$keyMapping['Control,Alt,P'] = @{
    KeypressHandled = { IsClassActive 'ConsoleWindowClass' };

    ## Console previous command starting with prefix
    Action =  { $autoIt.Send('{F8}') }
}



## Enable this key mapping to output the class name of the current window
## to the debug pane.
$keyMapping['Control,Alt,1'] = @{ 
   Action = { 
       $windowClass = [Windows.Automation.AutomationElement]::FocusedElement.Current.ClassName
       $windowTitle = (Get-Process -Id ([Windows.Automation.AutomationElement]::FocusedElement.Current.ProcessId)).MainWindowTitle
       Write-Debug "Current window class name: $windowClass"
       Write-Debug "Current window title: $windowTitle"
   } 
}
$keyMapping['Control,Alt,2'] = @{ Action = { DumpActiveWindow } }


$keyMapping['Control,Shift,C'] = @{ Action = { ClearBgShell } }

## Move the annoying mouse pointer out of your way
$keyMapping['Control,Alt,Z'] = @{ 
    Action = {
        [System.Windows.Forms.Cursor]::Position = (New-Object System.Drawing.Point 0,0) 
    }
}

$keyMapping['Control,Alt,E'] = @{
    KeypressHandled = { IsClassActive 'VsTextEditPane' };
    Action =  { EditFileInCurrentTab }
}
$keyMapping['Control,Alt,T'] = @{
    KeypressHandled = { IsWindowActive 'Product Studio' };
    Action =  { 
        $autoIt.ClipPut($autoit.WinGetTitle("","")) 
    }
}

## Control,Alt,C -- Convert selected text in an Outlook Message
## into a code sample
$keyMapping['Control,Alt,C'] = @{
    KeypressHandled = { IsClassActive '_WwG' };
    Action =  { 
        Start-Sleep -m 100
        SendKeys "%o"
        Start-Sleep -m 500
        SendString "ff"
        Start-Sleep -m 500
        SendString "Courier New"
        SendKeys "{ENTER}"
        Start-Sleep -m 500
        SendKeys "%o"
        Start-Sleep -m 500
        SendString "fs"
        Start-Sleep -m 500
        SendString "9"
        SendKeys "{ENTER}"
        Start-Sleep -m 500
    }
}

## Control,Alt,G -- Convert selected text in an Outlook Message
## into a green ("inline")
$keyMapping['Control,Alt,G'] = @{
    KeypressHandled = { IsClassActive '_WwG' };
    Action =  { 
        Start-Sleep -m 100
        SendKeys "%o"
        Start-Sleep -m 500
        SendString "fc"
        Start-Sleep -m 500
        SendString "m"
        Start-Sleep -m 500
        SendKeys "^{TAB}"
        Start-Sleep -m 500
        SendKeys "%r"
        SendString "0"
        SendKeys "{TAB}"
        SendString "153"
        SendKeys "{TAB}"
        SendString "0"
        SendKeys "{ENTER}"
    }
}

function Get-Input 
{ 
  param ($message = "Input : ", 
         $title = "Inputbox")

  $vbs = New-Object -com MSScriptControl.ScriptControl 
  $vbs.language = 'vbscript' 
  $vbs.addcode("function getInput() getInput = inputbox(`"$message`",`"$title`") end function") 
  $result = $vbs.Eval('getInput') 
  $result 
}

$stringMapping = @{}
$stringMapping[ "**hello" ] = { SayHello }
$stringMapping[ "**build" ] = { SendString $PSVersionTable.BuildVersion.Build }
$stringMapping[ "**prop " ] = 
{
    $template = @"
        /// <summary>
        /// Summary of what this property does
        /// </summary>
        public __TYPE__ __NAME__
        {
            get
            {
                return __NAMELOWER__;
            }
            set
            {
                __NAMELOWER__ = value;
            }
        }
        private __TYPE__ __NAMELOWER__;
"@

    $type,$name = (Get-Input -Message "Property type and name: ") -split " "
    
    $template = $template.Replace("__TYPE__", $type)
    $template = $template.Replace("__NAME__", $name)
    $template = $template.Replace("__NAMELOWER__", ($name.Substring(0,1).ToLower() + $name.Substring(1)))

    $autoIt.ClipPut($template)
}
